************************************************************************
*                                                                      *
*    Datei                                                             *
*      Isotherm                                                        *
*                                                                      *
*    Inhalt                                                            *
*      Sorp_F. Freundlich-Isotherme                                    *
*      Wird aufgerufen in:                                             *
*         W_calc                                                       *
*         G_calc                                                       *
*         Sorption                                                     *
*                                                                      *
*      Sorp_dc_F. Partielle Ableitung der Freundlich-Isotherme         *
*      Wird aufgerufen in:                                             *
*         Analyt                                                       *
*         DG_var                                                       *
*                                                                      *
*      Sorp_L. Langmuir-Isotherme                                      *
*      Wird aufgerufen in:                                             *
*         W_calc                                                       *
*         G_calc                                                       *
*         Sorption                                                     *
*                                                                      *
*      Sorp_dc_L. Partielle Ableitung der Langmuir-Isotherme           *
*      Wird aufgerufen in:                                             *
*         Analyt                                                       *
*         DG_var                                                       *
*                                                                      *
************************************************************************
*                                                                      *
*    Copyright                                                         *
*      Der in dieser Datei enthaltene Quellcode darf ohne Genehmigung  *
*      und weitere Vergtung weiterentwickelt werden. Diese Copyright  *
*      Erklrung darf um eigene Urherberrechtserklrungen erweitert    *
*      werden. Die Verwendung dieses Quelltextes fr Publikationen     *
*      ist, auch bei auszugsweiser Verwendung, grundstzlich Genehmi-  *
*      gungspflichtig (kostet nix, ich wrdes halt nur gerne wissen).  *
*                                                                      *
*      Der Programmautor weist darauf hin, da Software nicht fehler-  *
*      frei zu erstellen ist. Die Korrektheit des Quellcodes ist da-   *
*      her auf geeignete Weise zu berprfen.                          *
*                                                                      *
************************************************************************
*                                                                      *
*    Programmautor                                                     *
*      Dr. Rainer Sieger                                               *
*      Alfred-Wegener-Institut fr Meeres- und Polarforschung          *
*      Postfach 12 01 61                                               *
*      27515 Bremerhaven                                               *
*      e-mail: rsieger@AWI-Bremerhaven.de                              *
*                                                                      *
************************************************************************
*                                                                      *
*    letztes Editierdatum                                              *
*      16.3.94                                                         *
*                                                                      *
************************************************************************

**********************************************************************

      Real*8 Function Sorp_F(P1,P2,Konz)

**********************************************************************

      Real*8   P1, P2
      Real*8   Konz

**********************************************************************
*     Freundlich-Isotherme

      if (Konz.GT.1.D-250) then
        Sorp_F = P1*(Konz**P2)
      else
        Sorp_F = 0.D0
      end if

      End

**********************************************************************
**********************************************************************
**********************************************************************

      Real*8 Function Sorp_dc_F(P1,P2,Konz)

**********************************************************************

      Real*8  P1, P2
      Real*8  Konz

**********************************************************************
*     Freundlich-Isotherme, partiell abgeleitet

      if (Konz.GT.1.D-250) then
        Sorp_dc_F = P1*P2*Konz**(P2-1.D0)
      else
        Sorp_dc_F = 0.D0
      end if
      End

**********************************************************************
**********************************************************************
**********************************************************************

      Real*8 Function Sorp_L(P1,P2,Konz)

**********************************************************************

      Real*8   P1, P2
      Real*8   Konz

**********************************************************************
*     Langmuir-Isotherme

      if (Konz.GT.1.D-250) then
        Sorp_L = P1*Konz/(P2+Konz)
      else
        Sorp_L = 0.D0
      end if

      End

**********************************************************************
**********************************************************************
**********************************************************************

      Real*8 Function Sorp_dc_L(P1,P2,Konz)

**********************************************************************

      Real*8  P1, P2
      Real*8  Konz

**********************************************************************
*     Langmuir-Isotherme, partiell abgeleitet

      if (Konz.GT.1.D-250) then
        Sorp_dc_L = P1*P2/((P2+Konz)**2.D0)
      else
        Sorp_dc_L = 0.D0
      end if

      End
