************************************************************************
*                                                                      *
*    Datei                                                             *
*      Simulation.for                                                  *
*                                                                      *
*    Inhalt                                                            *
*      Simulation. Simulation des Stofftransport in porsen Medien     *
*      gekoppelt mit kinetisch gesteuerten Sorptionreaktionen.         *
*      Redox- und thermodynamische Reaktionen knnen bercksichtigt    *
*      werden.                                                         *
*      Eine ausfhrliche Dokumentation des Sourcecodes befindet sich   *
*      auf der Diskette im Verzeichnis \COTAM\SOURCE\ in der Datei     *
*      Readme.DVI. Fr den Ausdruck wird ein TeX-System bentigt.      *
*      TeX steht praktischen an allen universitren Rechenzentren      *
*      zur Verfgung. Es ist Public Domain Software und kann z.B.      *
*      ber ftp von diversen Fileserven bezogen werden. Eine gute      *
*      Adresse ist ftp.uni-stuttgart.de.                               *
*                                                                      *
************************************************************************
*                                                                      *
*    Copyright                                                         *
*      Der in dieser Datei enthaltene Quellcode darf ohne Genehmigung  *
*      und weitere Vergtung weiterentwickelt werden. Diese Copyright- *
*      Erklrung darf um eigene Urherberrechtserklrungen erweitert    *
*      werden. Die Verwendung dieses Quelltextes fr Publikationen     *
*      ist, auch bei auszugsweiser Verwendung, grundstzlich Genehmi-  *
*      gungspflichtig (kostet nix, ich wrdes halt nur gerne wissen).  *
*                                                                      *
*      Der Programmautor weist darauf hin, da Software nicht fehler-  *
*      frei zu erstellen ist. Die Korrektheit des Quellcodes ist da-   *
*      her auf geeignete Weise zu berprfen.                          *
*                                                                      *
************************************************************************
*                                                                      *
*    Programmautor                                                     *
*      Dr. Rainer Sieger                                               *
*      Alfred-Wegener-Institut fr Meeres- und Polarforschung          *
*      Postfach 12 01 61                                               *
*      27515 Bremerhaven                                               *
*      e-mail: rsieger@AWI-Bremerhaven.de                              *
*                                                                      *
************************************************************************
*                                                                      *
*    letztes Editierdatum                                              *
*      16.3.94                                                         *
*                                                                      *
************************************************************************

************************************************************************

      Subroutine Simulation

************************************************************************
************************************************************************
*     Deklaration der benutzten Variablen
*     Includes

      Include 'PHREEQE.CM'           ! PHREEQE-Variablen

      Include 'nT.inc'               ! max. # der Tracer
      Include 'nmax.inc'             ! max. # der REV
      Include 'nmess.inc'            ! max. # der Messwerte

************************************************************************
*     Parameter

      Integer*2  n_site, d_site
      Integer*2  Anzahl, nK
      Parameter (n_site = 2)          ! Two-Site-Isotherme
      Parameter (d_site = 4)          ! Two-Site-Isothermen-Parameter
      Parameter (nK     = nT)         ! Anzahl der Umsatzraten
      Parameter (Anzahl = 15)         ! # der Eingabeparameter

************************************************************************
*     Integer*2

      Integer*2 Ab_Zeichen            ! Abbruch der laufenden Simulation
      Integer*2 i, k, l               ! Laufvariablen
      Integer*2 Plot                  ! Kontroll-Flag
      Integer*2 Ausgabe               ! Zeitschritte zwischen 2 Ausgaben
      Integer*2 Schritt_Zaehler       ! If Schritt_Zhler = 0 => Ausgabe
      Integer*2 Abbruch               ! Kontroll-Flag
      Integer*2 Aufruf                ! Kontroll-Flag
      Integer*2 n                     ! # der Zellen
      Integer*2 MS(nT)                ! Masterspezie (Tracer) aktiv ?
      Integer*2 Site(nT)              ! # der Isothermen
      Integer*2 C_0_laden             ! Alte Konzentrationen laden ?
      Integer*2 Iso(nT)               ! Art der Isotherme
      Integer*2 PN(nT)                ! Phreeqe-Nr.
      Integer*2 PHREEQE_Aufruf        ! n. Aufruf von PHREEQE
      Integer*2 Anzahl_MS             ! # der aktiven Masterspezies
      Integer*2 Anzahl_MS_g           ! # der gesetzten Masterspezies
      Integer*2 max_Site              ! max. vorkommende Site
      Integer*2 Status                ! Dateistatus
      Integer*2 Rand                  ! Typ der Randbedingung
      Integer*2 Waits                 ! Warteschleife

************************************************************************
*     Integer

      Integer   Gesamt_iter           ! Gesamtanzahl der Iterationen
      Integer   n_r_iter              ! # N-R-Iterationen
      Integer   iter_max              ! max. # Iterationen

************************************************************************
*     Real

      Real      Messwerte(nmess,4)    ! Messwerte
      Real      Mess(nmess,4)         ! normierte Messwerte
      Real      Wert(Anzahl)          ! Datenvektor 1
      Real      SW(11,nT)             ! Datenvektor 2 (Sorpwert)
      Real      D_Rechenzeit          ! Rechenzeitbestimmung
      Real      DRZS, DRZE            ! Rechenzeitbestimmung
      Real      Typ                   ! GW oder Sedimente

************************************************************************
*     Real*8

      Real*8    Startzeit             ! Rechenzeitbestimmung
      Real*8    Endzeit               ! Rechenzeitbestimmung
      Real*8    Rechenzeit            ! Rechenzeitbestimmung
      Real*8    Dichte(3)             ! Dichte des Elementarvolumens
      Real*8    Porositaet(3)         ! Porositt des EV
      Real*8    Disp(3,nT)            ! Dispersionskoeffizient
      Real*8    Va(3)                 ! Abstandsgeschwindigkeit
      Real*8    dx(nmax)              ! Zellengre
      Real*8    C_0(nT)               ! Input-Konzentration
      Real*8    Saeulenlaenge         ! Sulenlnge
      Real*8    t_0                   ! Zpkt. f. Berechnung d. analy. L.
      Real*8    t                     ! Ablaufende Zeit
      Real*8    t_end                 ! Abgelaufene Zeit
      Real*8    t_max                 ! Endzeitpunkt
      Real*8    dt                    ! Zeitschrittlnge (numerische)
      Real*8    dt_Ausgabe            ! Ausgabezeitschrittlnge
      Real*8    eps                   ! Epsilon (Abbruchschranke)
      Real*8    dPV                   ! Porenvolumen
      Real*8    Rate(n_site,nT), R    ! Ratenkonstanten, Dummy
      Real*8    P(d_site,nT)          ! Sorptionsparameter
      Real*8    MG(nmax,3,nT)         ! Koeffizientenmatrix
      Real*8    alpha(n_site,nT)      ! Dummy
      Real*8    C(nmax,nT)            ! Konzentration in Zelle i
      Real*8    G(nmax), W(nmax)      ! Funktion und Inhomogenitt
      Real*8    Abweichung            ! Summe der Abweichungsquadrate
      Real*8    DG(nmax,3,nT)         ! Tridiagonale Funktionalmatrix
      Real*8    S(nmax,n_site,nT)     ! sorbierte Konzentration
      Real*8    TOTAL_input(MAXT)     ! Dummy fr PHREEQE Masterspezies
      Real*8    f_2S(nmax,n_site)     ! Dummy
      Real*8    C_nf(nT)              ! Normierungsfaktor fr C
      Real*8    S_nf(n_site,nT)       ! Normierungsfaktor fr S
      Real*8    Stoech(nT,nK)         ! Stchometrische Faktoren
      Real*8    UR(nmax,nK)           ! Umsatzraten
      Real*8    CNV(nmax)             ! C/N-Verhltnis

************************************************************************
*     Character

      Character*10  MS_Name(nT)       ! Masterspeziesnamen
      Character*37  Titel(2)          ! Titel des Datensatzes
      Character*12  DAT_File          ! Dateiname
      Character*12  PRN_File          ! Dateiname
      Character*12  PRX_File          ! Dateiname
      Character*12  PRX_File_neu      ! Dateiname
      Character*12  PAR_File          ! Dateiname
      Character*12  UMS_File          ! Dateiname
      Character*12  PHR_File          ! Dateiname
      Character*12  LPR_File          ! Dateiname
      Character*30  DAT_Pfad          ! Pfad fr DAT-File
      Character*30  PRN_Pfad          ! Pfad fr PRN-File
      Character*30  PRX_Pfad          ! Pfad fr PRX-File
      Character*30  PAR_Pfad          ! Pfad fr PAR-File
      Character*30  UMS_Pfad          ! Pfad fr UMS-File
      Character*30  PHR_Pfad          ! Pfad fr PHR-File
      Character*30  LPR_Pfad          ! Pfad fr LPR-File

************************************************************************
*     Logical

      Logical       Menue             ! Men gewnscht ?
      Logical       RC                ! ReDox Berechnungen ?
      Logical       PC                ! PHREEQE Berechnungen ?

************************************************************************
************************************************************************
*     Laden aller bentigten Daten aus COTAM.WRK

      call Load_Workfile(Status,Plot,Menue,Waits,  ! Kontroll-Flags
     *                   Mess,                     ! Messwerte
     *                   Titel,Wert,MS_Name,SW,    ! Daten
     *                   DAT_Pfad,DAT_File,        ! DAT
     *                   PAR_Pfad,PAR_File,        ! PAR
     *                   UMS_Pfad,UMS_File,        ! UMS
     *                   PHR_Pfad,PHR_File,        ! PHR
     *                   PRN_Pfad,PRN_File,        ! PRN
     *                   PRX_Pfad,PRX_File,        ! PRX
     *                   PRX_File_neu,             ! PRX
     *                   LPR_Pfad,LPR_File)        ! LPR

      if (Status.EQ.-1)  RETURN  ! keine Daten gefunden

************************************************************************
********************* Subroutine SIMULA ********************************
************************************************************************
*     Bildschirm Darstellung

      call Darstellung(Plot)

      Aufruf = 4
      if (Menue)  Aufruf = 3

************************************************************************
*     Setzen der Startzeit

      call Sekunden_lang(Startzeit)

************************************************************************
*     Variablen umbenennen und Initialisieren

      Saeulenlaenge = DBLE(Wert(3))  ! Lnge der Sule
      t_max         = DBLE(Wert(7))  ! Zu simulierende Zeit
      dt_Ausgabe    = DBLE(Wert(8))  ! Lnge eines Zeitschrittes
      C_0_laden     = IFIX(Wert(9))  ! Alte Konzentrationen laden
      dt            = DBLE(Wert(10)) ! numerische Zeitschrittlnge
      t_0           = DBLE(Wert(12)) ! Zpkt f. Berechnung d. analy. L.
      iter_max      = IFIX(Wert(13)) ! maximale # der Iterationen
      eps           = DBLE(Wert(14)) ! Epsilon (Abbruchschranke)
      Typ           = Wert(15)       ! Modelltyp

      Gesamt_iter   = 0              ! # der Gesamtiterationen

      do l=1,nT                      ! Normierungsfaktoren
        S_nf(1,l) = 1.D0
        S_nf(2,l) = 1.D0
      end do

      do i=1,nmax                    ! Dummyvektor (f-2*S)
        f_2S(i,1) = 0.D0
        f_2S(i,2) = 0.D0
      end do

      do l=1,nT                      ! kinetische Faktoren
        alpha(1,l) = 1.D0
        alpha(2,l) = 1.D0
      end do

      do l=1,nT                      ! PHREEQE-Nummern
        PN(l) = 0
      end do

************************************************************************
*     Anzahl der aktiven Masterspezies ermitteln

      call Aktive_Masterspezies(SW,MS,Anzahl_MS,Anzahl_MS_g)

************************************************************************
*     Koeffizientenmatrix bilden

      call Matrix_G(PAR_Pfad,PAR_File,     ! Pfad und Dateiname
     *              Wert,n,MS,Anzahl_MS,   ! Datenvektor1
     *              MG,Dichte,Porositaet,  ! Rckgabewerte
     *              Disp,Va,dx)            ! Rckgabewerte

************************************************************************
*     Auswahl der Randbedingung
*     Rand = 1 : Transmissionsrandbedingung
*     Rand = 2 : undurchlssiger Rand

      if(Va(3).GT.0.D0) then
        Rand = 1
      else
        Rand = 2
      end if

************************************************************************
*     Umbenennung der aktiven Sorptionsparameter

      do l=1,Anzahl_MS
        Iso(l)    = IFIX(SW(2,l))
        C_0(l)    = DBLE(SW(3,l))
        Rate(1,l) = DBLE(SW(4,l))
        P(1,l)    = DBLE(SW(5,l))
        P(2,l)    = DBLE(SW(6,l))
        Rate(2,l) = DBLE(SW(7,l))
        P(3,l)    = DBLE(SW(8,l))
        P(4,l)    = DBLE(SW(9,l))
        C_nf(l)   = DBLE(SW(10,l))
        PN(l)     = IFIX(SW(11,l))
      end do

      do l=1,Anzahl_MS
        if (C_nf(l).LE.0.D0)  C_nf(l) = 1.D0

        if (SW(1,l).GT.1.) then
          Rate(1,l) = -Rate(1,l)
          Rate(2,l) = -Rate(2,l)
        end if

        if(Iso(l).EQ.1) then
          Rate(1,l) = 1.0D30  ! fr Henry-Isotherme keine Kinetik
        end if

        if(Iso(l).EQ.2) then  ! fr Freundlich-Isotherme
          if((P(2,l).LE.1.D0) .AND. (P(2,l).GE.1.D0)) then
            P(2,l) = 1.D0+1.D-10
          end if
        end if
      end do

************************************************************************
*     Laden der PHREEQE-Database und des PHREEQE-Datensatzes

      PHREEQE_Aufruf = 0
      if (PHR_File.NE.'NONAME.PHR') then
        PC = .true.
        call read_DATABASE
        call read_DATA(PHR_Pfad,PHR_File)
        do l=1,MAXT
          TOTAL_input(l) = TOTAL(1,l)
        end do
      else
        PC = .false.
      end if

************************************************************************
*     Laden des REDOX-Datensatzes

      if (UMS_File.NE.'NONAME.UMS') then
        RC = .true.
        call UMS_Daten_laden(Typ,n,Anzahl_MS,Stoech,UR,CNV,
     *                       UMS_Pfad,UMS_File,PAR_Pfad,PAR_File)
      else
        RC = .false.
      end if

************************************************************************
*     Ermittlung der "Site" fr jede Isotherme

      do l=1,Anzahl_MS
        Site(l) = 0
        if (MS(l).GT.0) then
          if ((Rate(1,l).LT.0.D0) .OR. (Rate(1,l).GT.0.D0)) then
*           Henry-Isotherme
            if (Iso(l).EQ.1)  Site(l) = 1

*           Freundlich-Isotherme
            if ((Iso(l).EQ.2) .AND. (P(1,l).GT.0.D0))  Site(l) = 1

*           Langmuir-Isotherme
            if (Iso(l).EQ.3) then
              if ((Rate(1,l).LT.0.D0) .OR. (Rate(1,l).GT.0.D0)) then
                if ((P(1,l).GT.0.D0) .AND. (P(2,l).GT.0.D0)) Site(l) = 1
              end if
              if ((Rate(2,l).LT.0.D0) .OR. (Rate(2,l).GT.0.D0)) then
                if ((P(3,l).GT.0.D0) .AND. (P(4,l).GT.0.D0)) Site(l) = 2
              end if
            end if
          end if
        end if
      end do

************************************************************************
*     Berechnung des pro Zeitschritt durchflossenen Porenvolumens

      dPV = Va(1)*dt/Saeulenlaenge   ! durchflossenes PV/dt

************************************************************************
*     Berechnung der Anzahl der Schritte zwischen zwei Ausgaben

      Ausgabe = NINT(dt_Ausgabe/dt)  ! # der Zeitschritte

************************************************************************
*     Normierung der Messwerte

      Messwerte(1,1) = Mess(1,1)  ! # der Messwerte C(t,X)
      Messwerte(1,3) = Mess(1,3)  ! # der Messwerte C(x,t)

      if (Mess(1,2).LE.0.0)  Mess(1,2) = 1.0
      if (Mess(1,4).LE.0.0)  Mess(1,4) = 1.0

      do i=2, IFIX(Messwerte(1,1))+1
        Messwerte(i,1) = Mess(i,1)
        Messwerte(i,2) = Mess(i,2)/Mess(1,2)
      end do
      do i=2, IFIX(Messwerte(1,3))+1
        Messwerte(i,3) = Mess(i,3)
        Messwerte(i,4) = Mess(i,4)/Mess(1,4)
      end do

************************************************************************
*     Hintergrundkonzentration setzen

      call C_0_setzen(Plot,C_0_laden,
     *                n,Anzahl_MS,dt,t,
     *                C,S,
     *                PRX_Pfad,PRX_File,
     *                LPR_Pfad,LPR_File)

************************************************************************
*     P_1 und P_3 bestimmen (Rechenzeitersparnis)

      do l=1,Anzahl_MS
        if (MS(l).GT.0) then
          do k=1,Site(l)
            P(2*k-1,l) = Dichte(1)/Porositaet(1)*P(2*k-1,l)
          end do
        end if
      end do

************************************************************************
*     Defaultsorptionskonstanten berechnen

      do l=1,Anzahl_MS
        if (MS(l).GT.0) then
          do k=1,Site(l)
            r            = DABS(Rate(k,l)*dt)
            alpha(k,l) = r/(2.0D0+r)
            if (alpha(k,l).GT.0.99D0)  alpha(k,l) = 1.0D0
          end do
        end if
      end do

************************************************************************
*     Erffnen des Ausgabefiles

      call Ausgabefile(Plot,Titel,
     *                 DAT_Pfad,Dat_File,
     *                 PRN_Pfad,PRN_File)

************************************************************************
*     Berechnung der Konzentration in Lsung fr t=dt analytisch.
*     Berechnung der sorbierten Konzentration fr t=dt.
*     Die Berechnungen werden nur durchgefhrt, wenn die Konzentrationen
*     nicht aus einer Datei eingelesen wurden.

      if (C_0_laden.LT.1) then  ! wenn PRX-File nicht geladen wird
        do l=1,Anzahl_MS
          if (MS(l).GT.0) then
            call Analyt(n,Site,l,Iso,t_0,dx(1),C_0,Disp(1,l),Va(1),P,C)
            call Sorption(n,site,l,dx,alpha,P,Iso,C,f_2S,S)
          end if
        end do
      end if

************************************************************************
*     Setzen der Normierungsfaktoren

      do l=1,Anzahl_MS
        if (MS(l).GT.0) then
          if (S(1,1,l).GT.0.D0)  S_nf(1,l) = S(1,1,l)
          if (S(1,2,l).GT.0.D0)  S_nf(2,l) = S(1,2,l)
        end if
      end do

************************************************************************
*     Ergebnisausgabe vor dem ersten Schritt. Nullpunkt !

      call Ergebnisausgabe(1,Plot,Waits,Messwerte,SW,0,0,Startzeit,
     *                     0.,dPV,dt,t_max,t,dx,n,MS_Name,Site,
     *                     Anzahl_MS,MS,PN,C_0,C_nf,C,S_nf,S)

************************************************************************
*     Bilden der Tridiagonalmatrix (konstante Elemente)

      do l=1,Anzahl_MS
        if (MS(l).GT.0)  call DG_const(Rand,n,l,MG,DG)
      end do

************************************************************************
*
*     Zeitschleife, Beginn der eigentlichen Simulation
*
      Schritt_Zaehler = Ausgabe
      t_end           = t + dt

      call Sekunden_kurz(DRZS)

      do while (t.LE.t_max)
        do l=1,Anzahl_MS
         if (MS(l).GT.0) then
          n_r_iter = 0
          call W_calc(n,site,l,alpha,dx,Va,Disp,C_0,MG,P,Iso,S,C,f_2S,W)
          if (Site(l).EQ.0) then
*           Berechnung nur fr hydrodynamischen Fall
            call G_calc(Rand,n,site,l,alpha,dx,MG,P,Iso,f_2S,C,W,G)
            call DG_var(n,site,l,alpha,MG,P,Iso,f_2S,C,DG)
            call TRIDIAG(Rand,n,l,Abweichung,DG,G,C)
          else
*           Berechung mit Sorption (Newton-Raphson notwendig)
            Abweichung = 2.D0*eps
            do while ((n_r_iter.LE.iter_max) .AND. (Abweichung.GE.eps))
              n_r_iter = n_r_iter + 1
              call G_calc(Rand,n,site,l,alpha,dx,MG,P,Iso,f_2S,C,W,G)
              call DG_var(n,site,l,alpha,MG,P,Iso,f_2S,C,DG)
              call TRIDIAG(Rand,n,l,Abweichung,DG,G,C)
            end do
            Gesamt_iter = Gesamt_iter + n_r_iter
          end if
          call Sorption(n,site,l,dx,alpha,P,Iso,C,f_2S,S)
         end if
        end do

************************************************************************
*       REDOX

        if (RC) then
          call Redox(n,MS,Anzahl_MS,0.D0,dt,dx,Stoech,UR,CNV,C,
     *               UMS_Pfad,UMS_File)
        end if

************************************************************************
*       PHREEQE

        if (PC) then
          do i=1,n+2
            do l=1,MAXT
              TOTAL(1,l) = TOTAL_input(l)
            end do
            do l=1,Anzahl_MS
              if ((PN(l).GT.0) .AND. (PN(l).LE.MAXT)) then
                TOTAL(1,PN(l)) = C(i,l)
              end if
            end do

            PHREEQE_Aufruf = PHREEQE_Aufruf + 1
            call PHREEQE

            if (Plot.EQ.-1) then
              write(*,'(A)') ' '
              write(*,'(2(A,I4))')'Zelle : ',i,';  PHREEQE-Iter. :',iter
            end if
            do l=1,Anzahl_MS
              if ((PN(l).GT.0) .AND. (PN(l).LE.MAXT)) then
                C(i,l) = TOT(PN(l))
                if (Plot.EQ.-1) then  ! Ausgabe fr Debugzwecke
                  write(*,'(I3,2X,2(2X,1PE11.4))')
     *                     PN(l), TOTAL(1,PN(l)), TOT(PN(l))
                end if
              end if
            end do
          end do
        end if

************************************************************************
*       Abbruchkriterien

        Abbruch = 1
        l       = 1
        do while ((l.LE.Anzahl_MS) .AND. (Abbruch.EQ.1))
          if (MS(l).GT.0) then
            if (C_0(l).LE.0.D0) then
              if (C(n+1,l).GT.0.001)  Abbruch = 0
            else
              if (C(n+1,l)/C_nf(l).LT.0.999)  Abbruch = 0
            end if
          end if
          l = l + 1
        end do

        IF (Abbruch.EQ.0) then
          call Tastaturabfrage(Ab_Zeichen)
          IF (Ab_Zeichen.EQ.27) Abbruch = 1
          IF ((Ab_Zeichen.EQ.32).AND.(Plot.LT.7))  call warten_graphik
        end if

************************************************************************
*       Ergebnisausgabe

        if (Abbruch.LT.1) then
          if (Schritt_Zaehler.EQ.1) then
            call Sekunden_kurz(DRZE)
            D_Rechenzeit = (DRZE - DRZS)/Float(Ausgabe)
            call Sekunden_kurz(DRZS)
            call Ergebnisausgabe(2,Plot,Waits,Messwerte,SW,
     *                           Gesamt_iter,PHREEQE_Aufruf,
     *                           Startzeit,D_Rechenzeit,
     *                           dPV,dt,t_max,t,dx,n,
     *                           MS_Name,Site,Anzahl_MS,MS,PN,
     *                           C_0,C_nf,C,S_nf,S)
            Schritt_Zaehler = Ausgabe + 1
          end if

************************************************************************

          Schritt_Zaehler = Schritt_Zaehler - 1
          t               = t + dt
          t_end           = t
        else
          t = 2.*t_max   ! Vorzeitiger Abbruch
        end if
      end do   ! Ende der Zeitschleife

************************************************************************
*     Schreiben der Konzentrations-Verteilung in der Sule am Ende der
*     Simulationszeit in die Datei 'Filename.PRX'

      call Speicher_PRX(Plot,n,Anzahl_MS,t_end,dx,MS_Name,C,S,
     *                  PRX_Pfad,PRX_File_neu)

************************************************************************
*     Schreiben der aktuellen Umsatzraten-Verteilung in der Sule am
*     Ende der Simulationszeit in die Datei 'Filename.AUR'

      if (RC) then
        call Redox(n,MS,Anzahl_MS,t_end,dt,dx,Stoech,UR,CNV,C,
     *             UMS_Pfad,UMS_File)
      end if

************************************************************************
*     Ausgabe des letzten Berechnungsergebnisses

      call Sekunden_lang(Endzeit)
      Rechenzeit = Endzeit - Startzeit
      if (Rechenzeit.GT.120.D0)  call Piep

      call Ergebnisausgabe(Aufruf,Plot,Waits,Messwerte,SW,
     *                     Gesamt_iter,PHREEQE_Aufruf,
     *                     Startzeit,0.,
     *                     dPV,dt,t_max,t_end,dx,n,
     *                     MS_Name,Site,Anzahl_MS,MS,PN,
     *                     C_0,C_nf,C,S_nf,S)
      close(11)

************************************************************************
*     evtl. graphische Ausgabe der Sulenverteilungen fr 2 <= Plot <=3

      do l=1,Anzahl_MS
        if ((MS(l).GT.0).AND.(SW(1,l).LE.1.)) then
          if (S(1,1,l).GT.0.D0)  S_nf(1,l) = S(1,1,l)
          if (S(1,2,l).GT.0.D0)  S_nf(2,l) = S(1,2,l)
        end if
      end do

      if (Plot.EQ.2) then
        call Plotter(1,2,Waits,Anzahl_MS,MS,PN,Messwerte,t,
     *               t_max,dx,MS_Name,n,Site,C_nf,C_0,C,S_nf,S)
        call Plotter(Aufruf,2,Waits,Anzahl_MS,MS,PN,Messwerte,t_end,
     *               t_max,dx,MS_Name,n,Site,C_nf,C_0,C,S_nf,S)

        call calc_max_Site(Anzahl_MS,MS,Site,max_Site)
        do k=1,max_Site
          call Plotter(1,k+2,Waits,Anzahl_MS,MS,PN,Messwerte,t,
     *                 t_max,dx,MS_Name,n,Site,C_nf,C_0,C,S_nf,S)
          call Plotter(Aufruf,k+2,Waits,Anzahl_MS,MS,PN,Messwerte,t_end,
     *                 t_max,dx,MS_Name,n,Site,C_nf,C_0,C,S_nf,S)
        end do
      end if

      if (Plot.EQ.3) then
        call Plotter(1,8,Waits,Anzahl_MS,MS,PN,Messwerte,t,t_max,dx,
     *               MS_Name,n,Site,C_nf,C_0,C,S_nf,S)
        call Plotter(Aufruf,8,Waits,Anzahl_MS,MS,PN,Messwerte,t_end,
     *               t_max,dx,MS_Name,n,Site,C_nf,C_0,C,S_nf,S)
      end if

************************************************************************
*     Geordneter Rckzug zum Hauptprogramm

      if (Plot.EQ.7) then       ! Textmodus
        if (Aufruf.EQ.4) then
          call Pause(waits)
        else
          call Maske_o('Warte')
          call Taste
        end if
      end if
      if (Plot.EQ.9) then       ! Batchmodus
        if (Aufruf.EQ.4) then
          call Pause(waits)
        else
          call loesch_zeile(0,8,53)
          call warten(0,9)
          call cls
        end if
      end if

      call Cursor_on

      END

************************************************************************
************************************************************************
************************************************************************
*                                                                      *
*     Subroutinen, die in SIMULATION aufgerufen werden.                *
*                                                                      *
************************************************************************
************************************************************************
************************************************************************

      Subroutine Matrix_G(Pfad,Name,            ! Pfad und Dateiname
     *                    Wert,n,MS,Anzahl_MS,  ! Datenvektor1
     *                    MG,Dichte,Porositaet, ! Rckgabewerte
     *                    Disp,Va,dx)           ! Rckgabewerte

      Include 'nmax.inc'
      Include 'nT.inc'

      Integer*2      Anzahl
      Parameter      (Anzahl = 15)      ! # der Eingabeparameter

      Integer*2      i, l               ! Laufvariablen
      Integer*2      n                  ! # der Zellen
      Integer*2      MS(nT)             ! Masterspezie aktiv
      Integer*2      Anzahl_MS          ! # der Masterspezies
      Integer*2      Rand               ! Randbedingung ?

      Real           Wert(Anzahl)       ! Datenvektor 1
      Real           vW(25)             ! variable Parameter
      Real           Tiefe              ! Dummy

      Real*8         MG(nmax,3,nT)      ! Koeffizientenmatrix
      Real*8         dx(nmax)           ! Zellengre              = vW1
      Real*8         Dichte(3)          ! Dichte des REV           = vW2
      Real*8         Porositaet(3)      ! Porositt des REV        = vW3
      Real*8         Va(3)              ! Abstandsgeschwindigkeit  = vW4
      Real*8         Disp(3,nT)         ! Dispersionskoeffizient   = vW5
      Real*8         Diff(3,nT)         ! Diffusionskoeffizient    = vW6

      Real*8         t, v1, v2, D1, D2  ! Dummys
      Real*8         dt                 ! Zeitschrittlnge (numerische)

      Character*12   Name               ! Dateiname
      Character*30   Pfad               ! Dateipfad
      Character*42   Filename           ! vollstndiger Dateiname
      Character      Dummy

      Logical        Inhomogen          ! Berechnung fr homogene
      Logical        Existent_PAR       ! Existiert die Datei ?

************************************************************************

      n  = NINT(Wert(3)/Wert(11))  ! # der Zellen
      dt = DBLE(Wert(10))          ! numerische Zeitschrittlnge

      do i=1,n+2
        do l=1,Anzahl_MS
          mg(i,1,l) = 0.0D0
          mg(i,2,l) = 0.0D0
          mg(i,3,l) = 0.0D0
        end do
      end do

************************************************************************

      Inhomogen = .false.
      if (Name.NE.'NONAME.PAR') then
        call Verbinde_Pfad_Filename(Pfad,Name,Filename)
        inquire(File=Filename, Exist=Existent_PAR)
        if (Existent_PAR) Inhomogen = .true.
      end if

************************************************************************
*       Matrixaufbau fr inhomogene Situation.

      if (Inhomogen) then
        open(unit=5,file=Filename)
        read(5,'(A)') dummy
        read(5,'(A)') dummy
        read(5,'(A)') dummy

        do i=1,2
          read(5,*,END=1) Tiefe, (vW(l), l=1,5+Anzahl_MS)
          dx(i)         = DBLE(vW(1))        ! Zellengre
          Dichte(i)     = DBLE(vW(2))        ! Dichte
          Porositaet(i) = DBLE(vW(3))        ! Porositt des Sandes
          Va(i)         = DBLE(vW(4))        ! Abstandsgeschwindigkeit
          IF (Wert(15).GE.1.)  Va(i) = Va(i) * Porositaet(i)
          do l=1,Anzahl_MS
            if (MS(l).GT.0) then
              Diff(i,l) = DBLE(vW(l+5))         ! Diffusionskoeffizient
              IF (Wert(15).GE.1.)  Diff(i,l) = Diff(i,l) * Porositaet(i)
              Disp(i,l) = Va(i)*DBLE(vW(5))     ! Dispersion
              Disp(i,l) = Disp(i,l) + Diff(i,l) ! Dispersion
            end if
          end do
        end do

*  i = 1 ***************************************************************

        do l=1,Anzahl_MS
          if (MS(l).GT.0) then
            mg(1,1,l) = 0.D0
            mg(1,2,l) = - 2.D0*Disp(1,l)/dx(1) - Va(2)/2.D0
            mg(1,2,l) = mg(1,2,l) - 2.D0*Disp(2,l)/(dx(1) + dx(2))
            mg(1,3,l) = 2.D0*Disp(2,l)/(dx(1) + dx(2)) - Va(2)/2.D0
          end if
        end do

* i=2..n+1 *************************************************************

        i = 1
        do while(i.LT.nmax)
          read(5,*,END=1) Tiefe, (vW(l), l=1,5+Anzahl_MS)
          i             = i + 1
          dx(i+1)       = DBLE(vW(1))        ! Zellengre
*         Dichte(3)     = DBLE(vW(2))        ! Dichte
          Porositaet(3) = DBLE(vW(3))        ! Porositt des Sandes
          Va(3)         = DBLE(vW(4))        ! Abstandsgeschwindigkeit
          IF (Wert(15).GE.1.)  Va(3) = Va(3) * Porositaet(3)
          if ((i.EQ.n+1) .AND. (Va(3).LE.0.D0)) then
            Rand  = 2
            Va(3) = Va(2)
          end if
          do l=1,Anzahl_MS
            if (MS(l).GT.0) then
              Diff(3,l) = DBLE(vW(l+5))         ! Diffusionskoeffizient
              IF (Wert(15).GE.1.)  Diff(3,l) = Diff(3,l) * Porositaet(3)
              Disp(3,l) = Va(3)*DBLE(vW(5))     ! Dispersion
              Disp(3,l) = Disp(3,l) + Diff(3,l) ! Dispersion
            end if
          end do

          t  = dt/dx(i)
          v1 = Va(2)/4.D0
          v2 = Va(3)/4.D0
          do l=1,Anzahl_MS
            if (MS(l).GT.0) then
              D1       = Disp(2,l)/(dx(i-1) + dx(i))
              D2       = Disp(3,l)/(dx(i) + dx(i+1))
              mg(i,1,l) = -t*(D1 + v1)
              mg(i,2,l) = 1.D0 + t*(D1 + D2 - v1 + v2)
              mg(i,3,l) = -t*(D2 - v2)
            end if
          end do

          Va(2) = Va(3)
          do l=1,Anzahl_MS
            if (MS(l).GT.0) then
              Disp(2,l) = Disp(3,l)
              Diff(2,l) = Diff(3,l)
            end if
          end do
        end do
    1   close(5)

        n = i - 1
        if (Rand.EQ.2) Va(3) = 0.D0

************************************************************************
      else
************************************************************************
*       Matrixaufbau fr homogene Situation. Algorithmus teiloptimiert.

        do i=1,3
          Dichte(i)     = DBLE(Wert(1))        ! Dichte
          Porositaet(i) = DBLE(Wert(2))        ! Porositt des Sandes
          Va(i)         = DBLE(Wert(4))        ! Abstandsgeschwindigkeit
          do l=1,Anzahl_MS
            Disp(i,l) = DBLE(Wert(5))          ! Dispersivitt
            Diff(i,l) = DBLE(Wert(6))          ! Diffusionskoeffizient
            Disp(i,l) = Disp(i,l)*Va(i)        ! Dispersionskoeffizient
            Disp(i,l) = Disp(i,l) + Diff(i,l)  ! Dispersionskoeffizient
          end do
        end do

*  i = 1 ***************************************************************

        dx(1)    = DBLE(Wert(11))   ! Zellenweite
        mg(1,1,1) = 0.D0            ! wird spter nicht angesprochen
        mg(1,2,1) = - 2.D0*Disp(1,1)/dx(1)-Disp(2,1)/dx(1)-Va(2)/2.D0
        mg(1,3,1) = Disp(2,1)/dx(1) - Va(2)/2.D0

*       mg(1,2,1) = - Disp(2,1)/dx(1) - Va(2)       ! <== Flhler & Jury
*       mg(1,3,1) = Disp(2,1)/dx(1)                 ! <== (1983)

        do l=1,Anzahl_MS
          if (MS(l).GT.0) then
            mg(1,1,l) = mg(1,1,1)
            mg(1,2,l) = mg(1,2,1)
            mg(1,3,l) = mg(1,3,1)
          end if
        end do

*  i = 2,...,n+1  ******************************************************

        dx(2)    = DBLE(Wert(11))
        t        = dt/dx(2)
        mg(2,1,1) = -t*(Disp(2,1)/(dx(1) + dx(2)) + Va(1)/4.D0)
        mg(2,2,1) = 1.D0 + t*(2.D0*Disp(2,1)/(dx(1) + dx(2)))
        mg(2,3,1) = -t*(Disp(2,1)/(dx(1) + dx(2)) - Va(2)/4.D0)
        do i=2,n+1
          dx(i)  = DBLE(Wert(11))
          do l=1,Anzahl_MS
            if (MS(l).GT.0) then
              mg(i,1,l) = mg(2,1,1)
              mg(i,2,l) = mg(2,2,1)
              mg(i,3,l) = mg(2,3,1)
            end if
          end do
        end do

*  i = n+2  ************************************************************

        dx(n+2) = DBLE(Wert(11))
      end if

************************************************************************
*     Ausgabe der Koeffizientenmatrix zu Debug-Zwecken
*
*     do l=1,Anzahl_MS
*       call cls
*       do i=1,10
*         write(*,'(I2,3(2X,1PE10.3))') i,mg(i,1,l),mg(i,2,l),mg(i,3,l)
*       end do
*       call warten(0,20)
*     end do
*     call cls
*
************************************************************************

      End

************************************************************************
************************************************************************
************************************************************************

      Subroutine DG_const(Rand,n,l,MG,DG)

************************************************************************

      Include 'nmax.inc'
      Include 'nT.inc'

      Integer*2       n, i, l             ! # der Zellen, Laufvariable
      Integer*2       Rand                ! Typ der Randbedingung

      Real*8          MG(nmax,3,nT)       ! Koeffizientenmatrix
      Real*8          DG(nmax,3,nT)       ! partielle Ableitungen der
*                                         ! Funktionen

************************************************************************

      do i=1,n+1
        dg(i,1,l) = mg(i,1,l)
        dg(i,3,l) = mg(i,3,l)
      end do

      if (Rand.EQ.1) then
*       dg(n+2,0,l) =  1.D0   ! im Gleichungslser enthalten
        dg(n+2,1,l) = -2.D0
        dg(n+2,2,l) =  1.D0
      else
*       dg(n+2,0,l) = -1.D0   ! im Gleichungslser enthalten
        dg(n+2,1,l) =  0.D0
        dg(n+2,2,l) =  1.D0
      end if

      End

************************************************************************
************************************************************************
************************************************************************

      Subroutine W_calc(n,site,l,alpha,dx,Va,D,C_0,MG,P,Iso,S,C,f_2S,W)

************************************************************************

      Include 'nmax.inc'
      Include 'nT.inc'

      Integer*2  n_site, d_site
      Parameter (n_site = 2)            ! Two-Site-Isotherme
      Parameter (d_site = 4)            ! Two-Site-Isothermen-Parameter

      Integer*2  n, site(nT)            ! # der Zellen, Site der Tracer
      Integer*2  i, k, l                ! Laufvariablen
      Integer*2  Iso(nT)                ! Art der Isotherme der Tracer

      Real*8     MG(nmax,3,nT)          ! Koeffizientenmatrix
      Real*8     P1, P2                 ! Dummys
      Real*8     alpha(n_site,nT)       ! Dummy
      Real*8     dx(nmax)               ! ZellengrPe
      Real*8     Va(3)                  ! Abstandsgeschwindigkeit
      Real*8     D(3,nT)                ! Dispersionskoeffizient
      Real*8     C_0(nT)                ! Input-Konzentration
      Real*8     P(d_site,nT)           ! Isothermenparameter
      Real*8     C(nmax,nT)             ! Konzentration
      Real*8     W(nmax)                ! Inhomogenitten
      Real*8     S(nmax,n_site,nT)      ! sorbierte Konzentration
      Real*8     f_2S(nmax,n_site)      ! Dummy zur Beschleunigung
      Real*8     Sorp_F                 ! Freundlich-Isothermenfunktion
      Real*8     Sorp_L                 ! Langmuir-Isothermenfunktion

************************************************************************

      W(1) = C_0(l)*(2.D0*D(1,l)/dx(1) + Va(1))
*     W(1) = C_0(l)*Va(1) ! <== Flhler & Jury (1983)
      do i=2,n+1
        W(i) = C(i-1,l)*mg(i,1,l) + C(i,l)*(mg(i,2,l)-2.D0)
        W(i) = W(i) + C(i+1,l)*mg(i,3,l)
      end do
      W(n+2) = 0.0D0

      do k=1,Site(l)
        P1 = P(2*k-1,l)
        P2 = P(2*k,l)
        if (Iso(l).EQ.3) then  ! Langmuir-Isotherme
          do i=2,n+1
            f_2S(i,k) = Sorp_L(P1,P2,C(i,l)) - 2.0D0*s(i,k,l)
          end do
        else if (Iso(l).EQ.2) then  ! Freundlich-Isotherme
          do i=2,n+1
            f_2S(i,k) = Sorp_F(P1,P2,C(i,l)) - 2.0D0*s(i,k,l)
          end do
        else  ! Henry-Isotherme
          do i=2,n+1
            f_2S(i,k) = P1*C(i,l) - 2.0D0*s(i,k,l)
          end do
        end if
        do i=2,n+1
          W(i) = W(i) + alpha(k,l)*f_2S(i,k)
        end do
      end do

      End

************************************************************************
************************************************************************
************************************************************************

      Subroutine G_calc(Rand,n,site,l,alpha,dx,MG,P,Iso,f_2S,C,W,G)

************************************************************************

      Include 'nT.inc'
      Include 'nmax.inc'

      Integer*2       n_site, d_site
      Parameter      (n_site = 2)      ! Two-Site-Isotherme
      Parameter      (d_site = 4)      ! Two-Site-Isothermen-Parameter

      Integer*2      n, site(nT)       ! # der Zellen, Site der Tracer
      Integer*2      i, k, l           ! Laufvariablen
      Integer*2      Iso(nT)           ! Art der Isotherme der Tracer
      Integer*2      Rand              ! Type der Randbedingung

      Real*8         dx(nmax)          ! ZellengrPe
      Real*8         MG(nmax,3,nT)     ! Koeffizientenmatrix
      Real*8         P1, P2            ! Dummys
      Real*8         alpha(n_site,nT)  ! Dummy
      Real*8         P(d_site,nT)      ! Isothermenparameter
      Real*8         C(nmax,nT)        ! Konzentration
      Real*8         G(nmax)           ! Funktionen
      Real*8         f_2S(nmax,n_site) ! Dummy
      Real*8         W(nmax)           ! Inhomogenitten
      Real*8         Sorp_F            ! Freundlich-Isothermenfunktion
      Real*8         Sorp_L            ! Langmuir-Isothermenfunktion

************************************************************************

      G(1) = C(1,l)*mg(1,2,l) + C(2,l)*mg(1,3,l) + W(1)

      do i=2,n+1
        G(i) = C(i-1,l)*mg(i,1,l) + C(i,l)*mg(i,2,l)
        G(i) = G(i) + C(i+1,l)*mg(i,3,l) + W(i)
      end do

      if (Rand.EQ.1) then
        G(n+2) = C(n,l) - 2.D0*C(n+1,l) + C(n+2,l)   ! + W(n+2,l)
      else
        G(n+2) = -C(n,l) + C(n+2,l)                  ! + W(n+2,l)
      end if

      do k=1,Site(l)
        P1 = P(2*k-1,l)
        P2 = P(2*k,l)
        if (Iso(l).EQ.3) then ! Langmuir-Isotherme
          do i=2,n+1
            G(i) = G(i) + alpha(k,l)*Sorp_L(P1,P2,C(i,l))
          end do
        else if (Iso(l).EQ.2) then ! Freundlich-Isotherme
          do i=2,n+1
            G(i) = G(i) + alpha(k,l)*Sorp_F(P1,P2,C(i,l))
          end do
        else ! Henry-Isotherme
          do i=2,n+1
            G(i) = G(i) + alpha(k,l)*P1*C(i,l)
          end do
        end if
      end do

      End

************************************************************************
************************************************************************
************************************************************************

      Subroutine DG_var(n,site,l,alpha,MG,P,Iso,f_2S,C,DG)

************************************************************************

      Include 'nT.inc'
      Include 'nmax.inc'

      Integer*2      n_site, d_site
      Parameter     (n_site = 2)       ! Two-Site-Isotherme
      Parameter     (d_site = 4)       ! Two-Site-Isothermen-Parameter

      Integer*2      n, site(nT)       ! # der Zellen, Site der Tracer
      Integer*2      i, k, l           ! Laufvariablen
      Integer*2      Iso(nT)           ! Art der Isotherme der Tracer

      Real*8         MG(nmax,3,nT)     ! Koeffizientenmatrix
      Real*8         P1, P2            ! Dummys
      Real*8         alpha(n_site,nT)  ! Dummy
      Real*8         P(d_site,nT)      ! Isothermenparameter
      Real*8         C(nmax,nT)        ! Konzentration
      Real*8         DG(nmax,3,nT)     ! partielle Ableitungen der Fkt.
      Real*8         f_2S(nmax,n_site) ! Dummy
      Real*8         Sorp_dc_F         ! partielle Ableitung der F-Iso.
      Real*8         Sorp_dc_L         ! partielle Ableitung der L-Iso.

************************************************************************

      do i=1,n+1
        dg(i,2,l) = mg(i,2,l)
      end do

************************************************************************
*                                                                      *
*     dg(n+1,2,l) ist schon in Subroutine DG_const bestimmt worden.    *
*                                                                      *
************************************************************************

      do k=1,Site(l)
        P1 = P(2*k-1,l)
        P2 = P(2*k,l)
        if (Iso(l).EQ.3) then  ! Langmuir-Isotherme
          do i=2,n+1
            DG(i,2,l) = DG(i,2,l) + alpha(k,l)*Sorp_dc_L(P1,P2,C(i,l))
          end do
        else if (Iso(l).EQ.2) then  ! Freundlich-Isotherme
          do i=2,n+1
            DG(i,2,l) = DG(i,2,l) + alpha(k,l)*Sorp_dc_F(P1,P2,C(i,l))
          end do
        else  ! Henry-Isotherme
          do i=2,n+1
            DG(i,2,l) = DG(i,2,l) + alpha(k,l)*P1
          end do
        end if
      end do

      End

************************************************************************
************************************************************************
************************************************************************

      Subroutine Sorption(n,site,l,dx,alpha,P,Iso,C,f_2S,S)

************************************************************************

      Include 'nT.inc'
      Include 'nmax.inc'

      Integer*2       n_site, d_site
      Parameter      (n_site = 2)       ! Two-Site-Isotherme
      Parameter      (d_site = 4)       ! Two-Site-Isothermen-Parameter

      Integer*2      n, site(nT)        ! # der Zellen, Site der Tracer
      Integer*2      i, k, l            ! Laufvariablen
      Integer*2      Iso(nT)            ! Art der Isotherme

      Real*8         dx(nmax)          ! ZellengrPe
      Real*8         P1, P2, dummy     ! Dummys
      Real*8         alpha(n_site,nT)  ! Dummy
      Real*8         P(d_site,nT)      ! Isothermenparameter
      Real*8         C(nmax,nT)        ! Konzentration
      Real*8         S(nmax,n_site,nT) ! sorbierte Konzentration
      Real*8         f_2S(nmax,n_site) ! Dummy zur Beschleunigung
      Real*8         Sorp_F            ! Freundlich-Isothermenfunktion
      Real*8         Sorp_L            ! Langmuir-Isothermenfunktion

************************************************************************

      do k=1,Site(l)
        P1 = P(2*k-1,l)
        P2 = P(2*k,l)
        if (Iso(l).EQ.3) then  ! Langmuir-Isotherme
          do i=2,n+1
            dummy    = Sorp_L(P1,P2,C(i,l)) + f_2S(i,k)
            S(i,k,l) = S(i,k,l) + alpha(k,l)*dummy
          end do
        else if (Iso(l).EQ.2) then  ! Freundlich-Isotherme
          do i=2,n+1
            dummy    = Sorp_F(P1,P2,C(i,l)) + f_2S(i,k)
            S(i,k,l) = S(i,k,l) + alpha(k,l)*dummy
          end do
        else   ! Henry-Isotherme
          do i=2,n+1
            dummy    = P1*C(i,l) + f_2S(i,k)
            S(i,k,l) = S(i,k,l) + alpha(k,l)*dummy
          end do
        end if

************************************************************************
*    S(1,k,l) und S(n+2,k,l) werden aus Gradinenten berechnet. Sie
*    werden ausschlielich fr die graphische Darstellung betigt.

        S(1,k,l)   = dx(1)/dx(2)*(S(2,k,l) - S(3,k,l)) + S(2,k,l)
        S(n+2,k,l) = S(n+1,k,l)-dx(n+2)/dx(n+1)*(S(n,k,l)-S(n+1,k,l))
      end do

      End

************************************************************************
************************************************************************
************************************************************************

      Subroutine Ergebnisausgabe(Aufruf,Plot,Waits,Messwerte,SW,
     *                           Gesamt_iter,PHREEQE_Aufruf,
     *                           Startzeit,D_Rechenzeit,
     *                           dPV,dt,t_max,Zeit,dx,n,
     *                           MS_Name,Site,Anzahl_MS,MS,PN,
     *                           C_0,C_nf,C,S_nf,S)

************************************************************************

      Include 'nT.inc'
      Include 'nmax.inc'
      Include 'nmess.inc'

      Integer*2   n_site
      Parameter  (n_site = 2)

      Integer*2 l                      ! Laufvariable
      Integer*2 n                      ! # der Zellen
      Integer*2 Anzahl_MS              ! # der Masterspezies
      Integer*2 MS(nT)                 ! Tracer aktiv ?
      Integer*2 PN(nT)                 ! Phreeqe-Nr.
      Integer*2 RZ_std, RZ_min         ! noch betigte Rechenzeit
      Integer*2 PHREEQE_Aufruf         ! PHREEQE-Aufruf
      Integer*2 Plot                   ! soll geplottet werden ?
      Integer*2 Aufruf                 ! Aufruf des Plotter
      Integer*2 Site(nT)               ! Isotherme
      Integer*2 Waits                  ! Warteschleife

      Integer   Gesamt_iter            ! # der Gesamtiterationen

      Real      Messwerte(nmess,4)     ! Messwerte
      Real      Rechenzeit             ! Gesamtrechenzeit (sec.)
      Real      D_Rechenzeit           ! Rechenzeit pro Zeitschritt
      Real      RZ_Ende_std            ! noch bentigte Rechenzeit
      Real      SW(11,nT)              ! Datenvektor 2 (Sorpwert)

      Real*8    Startzeit              ! Rechenzeitbestimmung
      Real*8    Endzeit                ! Rechenzeitbestimmung
      Real*8    Zeit                   ! akt. Zeitpunkt
      Real*8    dPV, PV                ! Delta Porenvolumen, Porenvol.
      Real*8    dt                     ! numerische Zeitschrittlnge
      Real*8    t_max                  ! maximale Simulationszeit
      Real*8    dx(nmax)               ! Zellengre
      Real*8    C_0(nT)                ! Input-Konzentration
      Real*8    C_nf(nT)               ! Normierungsfaktor fr C
      Real*8    C_rel(nT)              ! relative Konzentration
      Real*8    C(nmax,nT)             ! Konzentration
      Real*8    C_aus(nT)              ! Formatierte Konzentrationen
      Real*8    S(nmax,n_site,nT)      ! sorbierte Stoffmenge
      Real*8    S_nf(2,nT)             ! Normierungsfaktor fr C

      Character*10  MS_Name(nT)        ! Masterspeziesnamen

************************************************************************

      PV = Zeit/dt*dPV
      do l=1,Anzahl_MS
        if (MS(l).GT.0) then
          C_rel(l) = C(n+1,l)/C_nf(l)
          C_aus(l) = C(n+1,l)
          if (DABS(C_aus(l)).LT.1.D-50) C_aus(l) = 0.
        else
          C_rel(l) = 0.
          C_aus(l) = 0.
        end if
      end do

      if (Plot.LE.3) then  ! Durchgangskurve
        call Plotter(Aufruf,1,Waits,Anzahl_MS,MS,PN,Messwerte,Zeit,
     *               t_max,dx,MS_Name,n,Site,C_nf,C_0,C,S_nf,S)
      end if

      if (Plot.EQ.4) then
        call Plotter(Aufruf,5,Waits,Anzahl_MS,MS,PN,Messwerte,Zeit,
     *               t_max,dx,MS_Name,n,Site,C_nf,C_0,C,S_nf,S)
      end if

      if (Plot.EQ.5) then
        do l=1,Anzahl_MS
          if ((MS(l).GT.0).AND.(SW(1,l).LE.1.)) then
            if (S(1,1,l).GT.0.D0)  S_nf(1,l) = S(1,1,l)
            if (S(1,2,l).GT.0.D0)  S_nf(2,l) = S(1,2,l)
          end if
        end do
        call Plotter(Aufruf,9,Waits,Anzahl_MS,MS,PN,Messwerte,Zeit,
     *               t_max,dx,MS_Name,n,Site,C_nf,C_0,C,S_nf,S)
      end if

      if (Plot.EQ.6) then
        call Plotter(Aufruf,11,Waits,Anzahl_MS,MS,PN,Messwerte,Zeit,
     *               t_max,dx,MS_Name,n,Site,C_nf,C_0,C,S_nf,S)
      end if

      if (Plot.EQ.7) then
        call Sekunden_lang(Endzeit)
        Rechenzeit  = Endzeit - Startzeit
        RZ_Ende_std = SNGL((t_max - Zeit)/dt)*D_Rechenzeit/3600.
        RZ_std      = INT(RZ_Ende_std)
        RZ_min      = INT((RZ_Ende_std - RZ_std)*60.0)

        call Schreibe_Wert(47,9,SNGL(Zeit))
        call Schreibe_Wert(47,10,SNGL(PV))
        call Schreibe_Wert(47,11,SNGL(C(n+1,1)))
        call Schreibe_Wert(47,12,SNGL(C_rel(1)))
        call gotoxy(47,14)
        write(*,'(I10)') Gesamt_iter
        call gotoxy(47,15)
        write(*,'(I10)') PHREEQE_Aufruf
        call Schreibe_Wert(47,17,Rechenzeit)
        call gotoxy(47,18)
        write(*,'(I7,A1,I2,$)')  RZ_std, ':', RZ_min
      end if

************************************************************************
*     Ausgabe der Werte in Datei. Die Division durch 10.D0 ist wegen
*     eines Fehlers im Fortran-Compilers FTN77/486 V2.69 notwendig.

      write(11,'(2(1X,F7.2,1X),20(2X,1PE10.3,1X,F7.4))')
     *      Zeit, PV, (C_aus(l), C_rel(l)/10.D0, l=1,Anzahl_MS)

************************************************************************

      End

************************************************************************
************************************************************************
************************************************************************

      Subroutine Aktive_Masterspezies(SW,MS,Anzahl_MS,Anzahl_MS_g)

      Include 'nT.inc'

      Integer*2   l             ! Laufvariable
      Integer*2   Anzahl_MS     ! # der aktiven Masterspezies
      Integer*2   Anzahl_MS_g   ! # der gesetzten Masterspezies
      Integer*2   MS(nT)        ! aktive Masterspezies

      Real        SW(11,nT)     ! Sorptionsparameter

************************************************************************
*     Aktive Masterspezies ermitteln

      do l=1,nT
        if (SW(1,l).GT.0.0) then
          MS(l) = 1
        else
          MS(l) = 0
        end if
      end do

************************************************************************
*     Anzahl der aktiven Masterspezies ermitteln

      l         = nT
      Anzahl_MS = 0
      do while((l.GE.1) .AND. (Anzahl_MS.EQ.0))
        if (MS(l).GT.0) Anzahl_MS = l
        l = l - 1
      end do

************************************************************************
*     Anzahl der gesetzten Masterspezies ermitteln

      l           = nT
      Anzahl_MS_g = 0
      do while((l.GE.1) .AND. (Anzahl_MS_g.EQ.0))
        if (SW(10,l).GT.0) Anzahl_MS_g = l
        l = l - 1
      end do

      End
