************************************************************************
*                                                                      *
*    Datei                                                             *
*      Tracer.for                                                      *
*                                                                      *
*    Inhalt                                                            *
*      Analyt                                                          *
*         Berechnung der analytischen Lsung als Startverteilung       *
*         mit vollstndiger Formel nach Kinzelbach (1986) oder         *
*         Flhler und Jury (1983).                                     *
*         Wird aufgerufen in:                                          *
*            Simulation                                                *
*                                                                      *
*      Tracerauswertung                                                *
*         Bestimmung von Durchgangskurven mit einem analytischen       *
*         Modell (Kinzelbach, 1987). Dient zur Auswertung von Tracer-  *
*         versuchen an Sulen. Bestimmung der Abstandsgeschwindigkeit  *
*         und der Dispersivitt.                                       *
*         Wird  aufgerufen in:                                         *
*            Main                                                      *
*            Start_Simulation                                          *
*                                                                      *
*      Tracer_analytisch                                               *
*         Berechnung der analytischen Lsung am Ende Sule mit voll-   *
*         stndiger Formel nach Kinzelbach (1986).                     *
*         Wird aufgerufen in:                                          *
*            Tracerauswertung                                          *
*                                                                      *
*      erfcf                                                           *
*         Komplementre Error-Function aus FLHLER und JURY (1983)     *
*         Wird aufgerufen in:                                          *
*            Analyt                                                    *
*                                                                      *
*      erfck                                                           *
*         Komplementre Error-Function aus KINZELBACH (1986)           *
*         Wird aufgerufen in:                                          *
*            Analyt                                                    *
*                                                                      *
*      erfcp                                                           *
*         Komplementre Error-Function aus PRESS et al. (1992)         *
*         Wird aufgerufen in:                                          *
*            Analyt                                                    *
*            Tracerauswertung                                          *
*            Tracer_analytisch                                         *
*                                                                      *
************************************************************************
*                                                                      *
*    Copyright                                                         *
*      Der in dieser Datei enthaltene Quellcode darf ohne Genehmigung  *
*      und weitere Vergtung weiterentwickelt werden. Diese Copyright- *
*      Erklrung darf um eigene Urherberrechtserklrungen erweitert    *
*      werden. Die Verwendung dieses Quelltextes fr Publikationen     *
*      ist, auch bei auszugsweiser Verwendung, grundstzlich Genehmi-  *
*      gungspflichtig (kostet nix, ich wrdes halt nur gerne wissen).  *
*                                                                      *
*      Der Programmautor weist darauf hin, da Software nicht fehler-  *
*      frei zu erstellen ist. Die Korrektheit des Quellcodes ist da-   *
*      her auf geeignete Weise zu berprfen.                          *
*                                                                      *
************************************************************************
*                                                                      *
*    Programmautor                                                     *
*      Dr. Rainer Sieger                                               *
*      Alfred-Wegener-Institut fr Meeres- und Polarforschung          *
*      Postfach 12 01 61                                               *
*      27515 Bremerhaven                                               *
*      e-mail: rsieger@AWI-Bremerhaven.de                              *
*                                                                      *
************************************************************************
*                                                                      *
*    letztes Editierdatum                                              *
*      16.3.94                                                         *
*                                                                      *
************************************************************************

************************************************************************

      Subroutine ANALYT(n,S,l,Iso,t,dz,C_0,D,Va,P,C)

************************************************************************

      Include 'nT.inc'
      Include 'nmax.inc'

      Integer*2       d_site
      Parameter      (d_site = 4)       ! Two-Site-Isothermen-Parameter

      Integer*2      i, k, l            ! Laufvariablen
      Integer*2      n                  ! # der Zellen
      Integer*2      S(nT)              ! Site der einzelnen Tracer
      Integer*2      Iso(nT)            ! Isotherme der einzenlen Tracer
      Integer*2      al                 ! Schalter

      Real*8         P(d_site,nT)       ! Isothermenparameter
      Real*8         P1, P2             ! Dummys
      Real*8         t                  ! t_0
      Real*8         retard             ! Retardationsfaktor
      Real*8         arg1, arg2, arg3   ! Dummys
      Real*8         Nenner, Dummy      ! Dummys
      Real*8         erfcf              ! komplementre Error-Funktion
      Real*8         erfck              ! komplementre Error-Funktion
      Real*8         erfcp              ! komplementre Error-Funktion
      Real*8         Sorp_dc_F          ! partielle Ableitung der F-Ios.
      Real*8         Sorp_dc_L          ! partielle Ableitung der L-Ios.
      Real*8         C(nmax,nT)         ! Konzentration
      Real*8         dz                 ! Zellenweite(1)
      Real*8         C_0(nT)            ! Input-Konzentration
      Real*8         D                  ! Dispersionskoeffizient(1)
      Real*8         Va                 ! Abstandsgeschwindigkeit(1)

************************************************************************
*     Bestimmung der Retardation

      retard = 1.0D0
      do k=1,S(l)
        P1 = P(2*k-1,l)
        P2 = P(2*k,l)
        if (Iso(l).EQ.1) then
          retard = retard + P1                       ! Henry-Isotherme
        else if (Iso(l).EQ.2) then
          retard = retard + Sorp_dc_F(P1,P2,C_0(l)) ! Freundlich-  "
        else
          retard = retard + Sorp_dc_L(P1,P2,C_0(l)) ! Langmuir-    "
        end if
      end do

************************************************************************
*     Schalter zwischen Kinzelbach (1986) und Fhler und Jury (1983)
*       al = 0; Flhler und Jury (1983)
*       al = 1; Kinzelbach (1986)
*       al = 2; Kinzelbach (1986) mit erfc aus Press et al. (1992)

      al = 2

************************************************************************
*     Berechnung der analytischen Lsung mit vollstndiger Formel
*     nach Flhler und Jury (1983)

      if (al.EQ.0) then
        C(1,l) = C_0(l)
        do i=2,n+1
          Nenner = 2.0D0*DSQRT(t*D*retard)
          if (Nenner.GT.0.0D0) then
            dummy  = retard*(i-1)*dz
            arg1   = (dummy - Va*t)/Nenner
            C(i,l) = 0.5D0*C_0(l)*(1.D0-erfcf(arg1))
          end if
        end do
        C(n+2,l) = C(n,l)
      end if

************************************************************************
*     Berechnung der analytischen Lsung mit vollstndiger Formel
*     nach Kinzelbach (1986).

      if (al.EQ.1) then
       do i=1,n+2
          Nenner = 2.0D0*DSQRT(t*D*retard)
          if (Nenner.GT.0.0D0) then
            dummy  = retard*(i-1)*dz
            arg1   = (dummy - Va*t)/Nenner
            arg2   = (i-1)*dz*Va/D
            if (arg2.GT.700.0)   arg2 = 0.D0
            arg3   = (dummy + Va*t)/Nenner
            C(i,l) = 0.5D0*C_0(l)*(erfck(arg1)+DEXP(arg2)*erfck(arg3))
          end if
        end do
      end if

************************************************************************
*     Berechnung der analytischen Lsung mit vollstndiger Formel
*     nach Kinzelbach (1986) mit erfc aus Press et al. (1992).

      if (al.EQ.2) then
       do i=1,n+2
          Nenner = 2.0D0*DSQRT(t*D*retard)
          if (Nenner.GT.0.0D0) then
            dummy  = retard*(i-1)*dz
            arg1   = (dummy - Va*t)/Nenner
            arg2   = (i-1)*dz*Va/D
            if (arg2.GT.700.0)   arg2 = 0.D0
            arg3   = (dummy + Va*t)/Nenner
            C(i,l) = 0.5D0*C_0(l)*(erfcp(arg1)+DEXP(arg2)*erfcp(arg3))
          end if
        end do
      end if

      END

************************************************************************
************************************************************************
************************************************************************
*                                                                      *
*     Subroutine Tracerauswertung                                      *
*                                                                      *
*     Bestimmung von Durchgangskurven mit einem analytischen Modell    *
*     (Kinzelbach, 1987). Dient zur Auswertung von Tracerversuchen     *
*     an Sulen. Bestimmung der Abstandsgeschwindigkeit und der        *
*     Dispersivitt.                                                   *
*                                                                      *
*     Programmautor: Rainer Sieger, AWI Bremerhaven                    *
*     letztes Editierdatum: 21.10.92                                   *
*                                                                      *
************************************************************************

      Subroutine Tracerauswertung(Menue,Waits,
     *                            Mess,Wert,MS_Name,SW,
     *                            Titel,
     *                            DAT_Pfad,DAT_File,
     *                            PRN_Pfad,PRN_File,
     *                            PRX_Pfad,PRX_File)

************************************************************************

      Include 'nmax.inc'
      Include 'nt.inc'
      Include 'nmess.inc'

      Integer*2       Anzahl, n_site
      Parameter      (Anzahl = 15)       ! # der Eingabeparameter
      Parameter      (n_site = 2)        ! Isotherme

************************************************************************

      Integer*2      i, l               ! Laufvariable
      Integer*2      n                  ! # der Zellen
      Integer*2      Anzahl_MS          ! # der Masterspezies
      Integer*2      Anzahl_MS_g        ! # der gesetzten Masterspezies
      Integer*2      MS(nT)             ! Tracer aktiv ?
      Integer*2      PN(nT)             ! Phreeqe-Nr.
      Integer*2      Site(nT)           ! Isotherme
      Integer*2      Waits              ! Warteschleife

      Real           SW(11,nT)          ! Sorptionsparameter
      Real           Wert(Anzahl)       ! Datenvektor
      Real           Messwerte(nmess,4) ! Messwerte
      Real           Mess(nmess,4)      ! normierte Messwerte

      Real*8         Zeit              ! aktueller Zeitpunkt
      Real*8         t_max             ! maximale Simulationszeit
      Real*8         dt                ! Ausgabezeitschrittlnge
      Real*8         Saeulenlaenge     ! Sulenlnge
      Real*8         Dichte            ! Dichte des Elementarvolumens
      Real*8         Porositaet        ! Porositt des EV
      Real*8         Disp              ! Dispersionskoeffizient
      Real*8         Diff              ! Diffusionskoeffizient
      Real*8         Va                ! Abstandsgeschwindigkeit
      Real*8         dx(nmax)          ! Gre der REV
      Real*8         retard(nT)        ! Retardationsfaktor
      Real*8         X                 ! Ort (Sulenende)
      Real*8         C_0(nT)           ! Input-Konzentration
      Real*8         C_nf(nT)          ! Normierungsfaktor fr C
      Real*8         C(nmax,nT)        ! Konzentration
      Real*8         S(nmax,n_site,nT) ! sorbierte Stoffmenge
      Real*8         S_nf(2,nT)        ! Normierungsfaktor fr C
      Real*8         arg1, arg2, arg3  ! Dummys
      Real*8         Nenner, Dummy     ! Dummys
      Real*8         erfcp             ! komplementre Error-Funktion

      Character*10   MS_Name(nT)       ! Namen der Masterspezies
      Character*37   Titel(2)          ! Titel des Datensatzes
      Character*12   DAT_File          ! Dateiname
      Character*12   PRN_File          ! Dateiname
      Character*12   PRX_File          ! Dateiname
      Character*30   DAT_Pfad          ! Pfad fr DAT-File
      Character*30   PRN_Pfad          ! Pfad fr PRN-File
      Character*30   PRX_Pfad          ! Pfad fr PRX-File

      Logical        Menue             ! Menue gewnscht ?

************************************************************************

      call Ausgabefile(1,Titel,
     *                 DAT_Pfad,DAT_File,
     *                 PRN_Pfad,PRN_File)

************************************************************************
*     Anzahl der aktiven Masterspezies ermitteln

      call Aktive_Masterspezies(SW,MS,Anzahl_MS,Anzahl_MS_g)

************************************************************************
*     Variablen umbenennen

      Dichte        = DBLE(Wert(1))        ! Dichte
      Porositaet    = DBLE(Wert(2))        ! Porositt des Sandes
      Saeulenlaenge = DBLE(Wert(3))        ! Lnge der Sule
      Va            = DBLE(Wert(4))        ! Abstandsgeschwindigkeit
      Disp          = DBLE(Wert(5))        ! Dispersivitt
      Diff          = DBLE(Wert(6))        ! Diffusionskoeffizient
      Disp          = Disp*Va + Diff       ! Dispersionskoeffizient
      t_max         = DBLE(Wert(7))        ! Zu simulierende Zeit
      dt            = DBLE(Wert(8))        ! Lnge eines Zeitschrittes

      n = Saeulenlaenge/DBLE(Wert(11))
      do i=1,n+2
        dx(i) = DBLE(Wert(11))             ! Zellengre
      end do

      do l=1,Anzahl_MS
        if (NINT(SW(2,l)).EQ.1) then
          retard(l) = 1.D0 + Dichte/Porositaet*DBLE(SW(5,l))
        else
          retard(l) = 1.D0
        end if
        C_0(l)    = DBLE(SW(3,l))
        C_nf(l)   = DBLE(SW(10,l))
        C(n+1,l)  = 0.D0
        S_nf(1,l) = 1.D0
        S_nf(2,l) = 1.D0
      end do

      X = Saeulenlaenge ! + 1.5D0*dx(1) ! Ort

************************************************************************
*     Normierung der Messwerte

      Messwerte(1,1) = Mess(1,1)  ! # der Messwerte C(t,X)
      Messwerte(1,3) = Mess(1,3)  ! # der Messwerte C(x,t)
      if (Mess(1,2).LE.0.0)  Mess(1,2) = 1.0
      if (Mess(1,4).LE.0.0)  Mess(1,4) = 1.0

      do i=2, NINT(Messwerte(1,1))
        Messwerte(i,1) = Mess(i,1)
        Messwerte(i,2) = Mess(i,2)/Mess(1,2)
      end do
      do i=2, NINT(Messwerte(1,3))
        Messwerte(i,3) = Mess(i,3)
        Messwerte(i,4) = Mess(i,4)/Mess(1,4)
      end do

************************************************************************

      call Plotter(1,1,Waits,Anzahl_MS,MS,PN,Messwerte,0.D0,t_max,dx,
     *             MS_Name,n,Site,C_nf,C_0,C,S_nf,S)

************************************************************************

      Zeit   = 0.D0
      do while (Zeit.LT.t_max)
        Zeit = Zeit + dt

        do l=1,Anzahl_MS
          call Tracer_analytisch(l,n,X,Zeit,retard(l),C_0(l),
     *                           Disp,Va,C)
        end do

        write(11,'(2(1X,F7.2),10(2X,F8.3,3X,F6.4))')
     *  Zeit, 0.0, (C(n+1,l), C(n+1,l)/C_0(l), l=1,Anzahl_MS)

        call Plotter(2,1,Waits,Anzahl_MS,MS,PN,Messwerte,Zeit,t_max,dx,
     *               MS_Name,n,Site,C_nf,C_0,C,S_nf,S)
      end do

************************************************************************

      close(11)

************************************************************************
*     Berechnung der analytischen Lsung mit vollstndiger Formel
*     nach Kinzelbach (1986) und schreiben der Konzentrations-Verteilung
*     in der Sule am Ende der Simulationszeit in die Datei
*     'Filename.PRX' und graphische Ausgabe.

      do l=1,Anzahl_MS
        do i=1,n+2
          Nenner = 2.0D0*DSQRT(t_max*Disp*retard(l))
          if (Nenner.GT.0.0D0) then
            dummy  = retard(l)*(i-1)*dx(i)
            arg1   = (dummy - Va*t_max)/Nenner
            arg2   = (i-1)*dx(i)*Va/Disp
            if (arg2.GT.700.0)   arg2 = 0.D0
            arg3   = (dummy + Va*t_max)/Nenner
            C(i,l) = 0.5D0*C_0(l)*(erfcp(arg1)+DEXP(arg2)*erfcp(arg3))
          end if
        end do
      end do

      call Speicher_PRX(1,n,Anzahl_MS,t_max,dx,MS_Name,C,S,
     *                  PRX_Pfad,PRX_File)

************************************************************************

      l = 4
      if (Menue)  l=3
      call Plotter(l,1,Waits,Anzahl_MS,MS,PN,Messwerte,Zeit,t_max,dx,
     *             MS_Name,n,Site,C_nf,C_0,C,S_nf,S)

************************************************************************

      End

************************************************************************
************************************************************************
************************************************************************
*                                                                      *
*     Subroutine Tracer_analytisch                                     *
*                                                                      *
*     Berechnung der analytischen Lsung am Ende Sule mit vollstn-   *
*     diger Formel nach Kinzelbach (1986).                             *
*                                                                      *
************************************************************************

      Subroutine Tracer_analytisch(l,n,X,Zeit,retard,C_0,D,Va,C)

************************************************************************

      Include 'nT.inc'
      Include 'nmax.inc'

      Integer*2      l                  ! Laufvariable
      Integer*2      n                  ! # der Zellen

      Real*8         Zeit               ! Zeit
      Real*8         t                  ! Dummy
      Real*8         arg1, arg2, arg3   ! Dummys
      Real*8         Nenner             ! Dummy
      Real*8         C(nmax,nT)         ! Konzentration
      Real*8         C_0                ! Input-Konzentration
      Real*8         D                  ! Dispersionskoeffizient
      Real*8         Va                 ! Abstandsgeschwindigkeit
      Real*8         retard             ! Retardationsfaktor
      Real*8         X                  ! Ort (Sulenende)

      Real*8         erfcp              ! komplementre Error-Funktion

************************************************************************

      t    = Zeit/retard
      arg2 = X*Va/D
      if (arg2.GT.700.D0)  arg2 = 0.D0

      Nenner   = 2.0D0*DSQRT(t*D)
      arg1     = (X - Va*t)/Nenner
      arg3     = (X + Va*t)/Nenner
      C(n+1,l) = 0.5D0*C_0*(erfcp(arg1)+DEXP(arg2)*erfcp(arg3))

************************************************************************

      END

************************************************************************
************************************************************************
************************************************************************
*     Komplementre Error-Function aus KINZELBACH (1986)

      Real*8 FUNCTION erfck(x)

************************************************************************

      Real*8   x                        ! Argument
      Real*8   b1, b2, b3, b4, b5, c    ! Dummys

************************************************************************

      b1 =  0.254829592D0
      b2 = -0.284496736D0
      b3 =  1.421413741D0
      b4 = -1.453152027D0
      b5 =  1.061405429D0
      c  =  1.0D0/(1.0D0+0.327591117D0 * Dabs(x))

      erfck = dexp(-x*x)*(b1*c+b2*c**2.0+b3*c**3.0+b4*c**4.0+b5*c**5.0)
      if (x.LT.0.0) then
        erfck = 2.0D0 - erfck
      end if

      END

************************************************************************
************************************************************************
************************************************************************
*     Komplementre Error-Function aus PRESS et al. (1992)

      Real*8 FUNCTION erfcp(x)

************************************************************************

      Real*8   x             ! Argument
      Real*8   b(10), z, t   ! Dummys

************************************************************************

      b(1)  =  1.26551223D0
      b(2)  =  1.00002368D0
      b(3)  =  0.37409196D0
      b(4)  =  0.09678418D0
      b(5)  = -0.18628806D0
      b(6)  =  0.27886807D0
      b(7)  = -1.13520398D0
      b(8)  =  1.48851587D0
      b(9)  = -0.82215223D0
      b(10) =  0.17087277D0
      z     = DABS(x)
      t     = 1.D0/(1.D0 + 0.5D0*z)
      erfcp = t*DEXP(-z*z-b(1)+t*(b(2)+t*(b(3)+t*(b(4)+t*(b(5)+t*(b(6)+
     *        t*(b(7)+t*(b(8)+t*(b(9)+t*b(10))))))))))

      if (x.LT.0.0) then
        erfcp = 2.0D0 - erfcp
      end if

      END

************************************************************************
*     Komplementre Error-Function nach FLHLER und JURY (1983)

      Real*8 FUNCTION erfcf(x)

************************************************************************

      Real*8   x             ! Argument
      Real*8   RA, EA, R, B  ! Dummys

************************************************************************

      RA = DABS(x)
      if (RA.GT.5.D0) then
        erfcf = 1.D0
      else
        R     = RA/X
        EA    = DEXP(-RA*RA)
        B     = 1.D0/(1.D0 + 0.47047D0*RA)
        B     = 1.D0-EA*(B*(0.3480242D0-B*(0.0956798D0-B*0.7478556D0)))
        erfcf = R*B
      end if

      END
